#!/bin/bash

SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
. $SCRIPT_DIR/common.sh "$@"


echo "-- INSTALLING MESOS ------------------------------------------------------------"

export MESOS_VERSION=1.3.0

saved_dir=`pwd`
function returned_to_saved_dir() {
     cd $saved_dir
}
trap returned_to_saved_dir 15
trap returned_to_saved_dir EXIT

echo " - Changing to temp directory"
mkdir -p /tmp/mesos_setup
cd /tmp/mesos_setup

echo " - Downloading mesos-$MESOS_VERSION"
wget http://www.apache.org/dist/mesos/$MESOS_VERSION/mesos-$MESOS_VERSION.tar.gz > /tmp/mesos_install_log 2>&1
if [[ $? != 0 ]]; then
    echo " -> Failed to downolad mesos-$MESOS_VERSION from http://www.apache.org/d. Trying to download from niceideas.ch"
    wget http://niceideas.ch/mes/mesos-$MESOS_VERSION.tar.gz >> /tmp/mesos_install_log 2>&1
    fail_if_error $? "/tmp/mesos_install_log" -1
fi

echo " - Extracting mesos-$MESOS_VERSION"
tar -zxf mesos-$MESOS_VERSION.tar.gz > /tmp/mesos_install_log 2>&1
fail_if_error $? "/tmp/mesos_install_log" -2

echo " - Configuring source tree (preparing build)"
cd ./mesos-$MESOS_VERSION/
mkdir build
cd build
../configure --prefix=/usr/local/lib/mesos-$MESOS_VERSION/ > /tmp/mesos_install_log 2>&1
fail_if_error $? "/tmp/mesos_install_log" -3
echo " - Building source tree (!! this takes a long time, up to 30 mins !!)"
make -j 2 > /tmp/mesos_install_log 2>&1
fail_if_error $? "/tmp/mesos_install_log" -4

# Only running tests once on master
# FIXME : some of them fail for now :-(
#if [[ $IS_MASTER == 1 ]]; then
#    echo " - Running test suite (!! this takes a long time, up to 30 mins !!)"
#    make -j 2 check > /tmp/mesos_install_log 2>&1
#    fail_if_error $? "/tmp/mesos_install_log" -5
#fi

echo " - Installing mesos"
sudo make install > /tmp/mesos_install_log 2>&1
fail_if_error $? "/tmp/mesos_install_log" -6

# installation tests

echo " - Basic environment setup "
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/usr/local/lib/
sudo bash -c 'echo -e "\nexport LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/usr/local/lib/" > /etc/profile'
sudo ldconfig
sudo mkdir -p /var/lib/mesos
sudo chmod 777 /var/lib/mesos

echo " - Registering test cleaning traps"
export MESOS_MASTER_PROC_ID=-1
export MESOS_AGENT_PROC_ID=-1
function check_stop_mesos(){
    if [[ $MESOS_MASTER_PROC_ID != -1 ]]; then
        echo " - Stopping Mesos Master !!"
        kill -15 $MESOS_MASTER_PROC_ID
    fi
    if [[ $MESOS_AGENT_PROC_ID != -1 ]]; then
        echo " - Stopping Mesos Agent !! "
        kill -15 $MESOS_AGENT_PROC_ID
    fi
}
trap check_stop_mesos 15
trap check_stop_mesos EXIT

echo " - Starting Mesos Master"
/usr/local/lib/mesos-$MESOS_VERSION/sbin/mesos-master  --ip=127.0.0.1 --work_dir=/var/lib/mesos > /tmp/mesos_master_log 2>&1 &
export MESOS_MASTER_PROC_ID=$!

echo " - Checking Mesos master startup"
sleep 2
if [[ `ps -e | grep $MESOS_MASTER_PROC_ID` == "" ]]; then
    echo " !! Failed to start Mesos Master !!"
    exit -8
fi

echo " - Starting Mesos Agent"
/usr/local/lib/mesos-$MESOS_VERSION/sbin/mesos-agent  --master=127.0.0.1:5050 --no-systemd_enable_support --work_dir=/var/lib/mesos > /tmp/mesos_agent_log 2>&1 &
export MESOS_AGENT_PROC_ID=$!

echo " - Checking Mesos Agent startup"
sleep 2
if [[ `ps -e | grep $MESOS_AGENT_PROC_ID` == "" ]]; then
    echo " !! Failed to start Mesos Agent !!"
    exit -9
fi

# Visit the Mesos web page.
#$ http://127.0.0.1:5050


# Only running tests once on master
# FIXME : some of them fail for now :-(
#if [[ $IS_MASTER ]]; then
#    echo " - Run Java tests framework "
#    ./src/examples/java/test-framework 127.0.0.1:5050 > /tmp/mesos_install_log 2>&1
#    fail_if_error $? "/tmp/mesos_install_log" -11
#
#    echo " - Run Python framework "
#    ./src/examples/python/test-framework 127.0.0.1:5050 > /tmp/mesos_install_log 2>&1
#    fail_if_error $? "/tmp/mesos_install_log" -12
#fi

echo " - Stopping Mesos Agent"
kill -15 $MESOS_AGENT_PROC_ID
export MESOS_AGENT_PROC_ID=-1

echo " - Stopping Mesos Master"
kill -15 $MESOS_MASTER_PROC_ID
export MESOS_MASTER_PROC_ID=-1


echo " - Cleaning build files"
make clean > /tmp/mesos_install_log 2>&1
fail_if_error $? "/tmp/mesos_install_log" -7

echo " - Cleaning build folder"
cd $saved_dir
sudo rm -Rf /tmp/mesos_setup > /tmp/mesos_install_log 2>&1
fail_if_error $? "/tmp/mesos_install_log" -10


returned_to_saved_dir





