#!/bin/bash

# Use this script to recreate a vagrant box from a base debian box

set -o errexit
set -o nounset
set -o xtrace

VM='temp-vagrant-box'
DIR=target
box="niceideas-debian-stretch64.box"

wget https://vagrantcloud.com/debian/boxes/stretch64/versions/9.0.0/providers/virtualbox.box
# => virtualbox.box

tar xvfz virtualbox.box

rm -f ./metadata.json
rm -f ./box.ovf
rm -f ./Vagrantfile




# resize disk
vboxmanage clonehd "stretch.vmdk" "temp_disk.vdi" --format vdi
vboxmanage modifyhd "temp_disk.vdi" --resize 15000

rm -f ./stretch.vmdk


# Delete VM if it already exists, otherwise don't stop script

VBoxManage unregistervm $VM --delete || true


# Create new VM and set system settings 

VBoxManage createvm --name $VM --ostype "Debian_64" --register


VBoxManage storagectl $VM --name "SATA Controller" --add sata --controller IntelAHCI

# Add an IDE controller with a DVD drive attached, and the install ISO inserted into the drive:

VBoxManage storagectl $VM --name "IDE Controller" --add ide
VBoxManage storageattach $VM --storagectl "SATA Controller" --port 0 --device 0 --type hdd --medium temp_disk.vdi

# Misc system settings.
 
VBoxManage modifyvm $VM --ioapic on
VBoxManage modifyvm $VM --boot1 disk --boot2 dvd --boot3 none --boot4 none
VBoxManage modifyvm $VM --memory 2048 --vram 128
VBoxManage modifyvm $VM --cpus 2


# Mount VirtualBox Guest Additions CD-ROM

# JKE hack : in case the "additions" medium could not be found
VBoxManage storageattach $VM --storagectl "IDE Controller" --port 0 --device 0 --type dvddrive --medium additions || \
	    VBoxManage storageattach $VM --storagectl "IDE Controller" --port 0 --device 0 --type dvddrive --medium ~/.config/VirtualBox/VBoxGuestAdditions_*.iso || \
	    VBoxManage storageattach $VM --storagectl "IDE Controller" --port 0 --device 0 --type dvddrive --medium /usr/share/virtualbox/VBoxGuestAdditions.iso


# Forward ports. This allows us to SSH to the VM by ssh'ing to localhost:$host_ssh_port
host_ssh_port=3356
guest_ssh_port=22
if ! VBoxManage showvminfo $VM | grep sshPortForward; then
	VBoxManage modifyvm $VM --natpf1 "sshPortForward,tcp,127.0.0.1,$host_ssh_port,,$guest_ssh_port"
fi

# Power up the VM
VBoxManage startvm $VM


# TODO I still need here to 
# 1. remove the /vagrant/ssh/.authorized_kes entries
# 2. And uncomment PassordLoginPermitted (or something alike) from /etc/ssh/sshd_config
# 3. Restart sshd through systemd

# Run Expect script to SSH into the VM and prepare it for Vagrant
expect vagrant-box-create.ex

# Resize disk
ssh vagrant@127.0.0.1 -p $host_ssh_port -o LogLevel=FATAL -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -o IdentitiesOnly=yes -i ../vm-key < vagrant-box-prepare.sh

# Reboot
VBoxManage controlvm $VM acpipowerbutton
until VBoxManage showvminfo --machinereadable $VM | grep -q ^VMState=.poweroff.; do echo "Waiting for VM to shut down..."; sleep 2; done

# Power up the VM
VBoxManage startvm $VM

# Provision the VM
ssh vagrant@127.0.0.1 -p $host_ssh_port -o LogLevel=FATAL -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -o IdentitiesOnly=yes -i ../vm-key < vagrant-box-provision.sh

# Shutdown the VM. acpid is now installed, so we can shutdown nicely

VBoxManage controlvm $VM acpipowerbutton
until VBoxManage showvminfo --machinereadable $VM | grep -q ^VMState=.poweroff.; do echo "Waiting for VM to shut down..."; sleep 2; done

# Eject the DVD

VBoxManage storageattach $VM --storagectl "IDE Controller" --port 0 --device 0 --type dvddrive --medium none

# Export the VM as a Vagrant Box

test -e $box && rm $box
vagrant package --base $VM --output $box

echo "*** Complete. Output is in $box ***"

VBoxManage unregistervm $VM --delete 

rm -Rf temp_disk.vdi
rm -Rf 




